@@ -236,7 +236,7 @@ template<class T> void CCP4MAPfile::import_xmap( Xmap<T>& xmap ) const
     index = 0;
     CMap_io::ccp4_cmap_read_section( file, &section[0] );
     if ( dmode == 0 ) for ( int i = n1-1; i >= 0; i-- )  // deal with byte maps
-      section[i] = float(((unsigned char*)&section[0])[i]);
+      section[i] = float(((signed char*)&section[0])[i]);
     for ( g[1] = gfms0[1]; g[1] <= gfms1[1]; g[1]++ ) {
       for ( g[0] = gfms0[0]; g[0] <= gfms1[0]; g[0]++ ) {
 	x.set_coord( Coord_grid( g[orderxyz[0]], g[orderxyz[1]],
@@ -462,6 +462,111 @@ template<class T> void CCP4MAPfile::export_nxmap( const NXmap<T>& nxmap )
 }
 
 
+/*! Import a complete Xmap object. The supplied Xmap object is
+  examined, and if any of the parameters (spacegroup, cell, or
+  grid_sampling) are unset, then they will be set using values from the
+  file. The data is the imported from the file.
+
+  If the spacegroups mismatch, the resulting map will obey its
+  spacegroup symmetry, but no expansion will be performed if the file
+  has a higher symmetry and only holds an asymmetric unit.
+
+  This version performs a check on the symmatry of the inport map
+  file. If the import map contains less than a complete ASU, or if it
+  contains more than an ASU and symmetry related positions contain
+  inconsistent values, then an error is returned.
+
+  This function is only available for float or double maps.
+
+  \param xmap The Xmap to be imported.
+  \return The ASU check flag
+*/
+template<class T> CCP4MAPfile::ASUerror CCP4MAPfile::import_xmap_check_asu( Xmap<T>& xmap, T missing ) const
+{
+  if ( mode != READ )
+    Message::message( Message_fatal( "CCP4MAPfile: no file open for read" ) );
+
+  // first check if the HKL_info params are already set
+  Spacegroup    s = xmap.spacegroup();
+  Cell          c = xmap.cell();
+  Grid_sampling r = xmap.grid_sampling();
+  // import any missing params
+  if ( s.is_null() ) s = spacegroup_;
+  if ( c.is_null() ) c = cell_;
+  if ( r.is_null() ) r = grid_sam_;
+  xmap.init( s, c, r );
+
+  // set missing flag
+  T flag;
+  Util::set_null( flag );
+  xmap = flag;
+  ftype maxerr = 0.0;
+
+  // read the file header
+  int orderfms[3], orderxyz[3], dim[3], gfms0[3], gfms1[3];
+  CMap_io::CMMFile* file = (CMap_io::CMMFile*)CMap_io::ccp4_cmap_open( filename.c_str(), O_RDONLY );
+  if ( file == NULL ) Message::message( Message_fatal( "CCP4MAPfile: import_xmap - File missing or corrupted: "+filename ) );
+  CMap_io::ccp4_cmap_get_order( file, orderfms );
+  CMap_io::ccp4_cmap_get_dim( file, dim );
+  CMap_io::ccp4_cmap_get_origin( file, gfms0 );
+  int dmode = CMap_io::ccp4_cmap_get_datamode( file );
+  if ( dmode != 0 && dmode != 2 )
+    Message::message( Message_fatal( "CCP4CCP4MAPfile: unsupported data mode" ) );
+
+  // get grid bound and axis order
+  for ( int i = 0; i < 3; i++ ) gfms1[i] = gfms0[i] + dim[i] - 1;
+  for ( int i = 0; i < 3; i++ ) orderxyz[orderfms[i]-1] = i;
+
+  // read the map data
+  int n0 = (gfms1[0]-gfms0[0]+1);
+  int n1 = n0 * (gfms1[1]-gfms0[1]+1);
+  std::vector<float> section( n1 );
+  int index, g[3];
+  Xmap_base::Map_reference_coord x( xmap );
+  for ( g[2] = gfms0[2]; g[2] <= gfms1[2]; g[2]++ ) {
+    index = 0;
+    CMap_io::ccp4_cmap_read_section( file, &section[0] );
+    if ( dmode == 0 ) for ( int i = n1-1; i >= 0; i-- )  // deal with byte maps
+      section[i] = float(((unsigned char*)&section[0])[i]);
+    for ( g[1] = gfms0[1]; g[1] <= gfms1[1]; g[1]++ ) {
+      for ( g[0] = gfms0[0]; g[0] <= gfms1[0]; g[0]++ ) {
+	x.set_coord( Coord_grid( g[orderxyz[0]], g[orderxyz[1]],
+				 g[orderxyz[2]] ) );
+	T oldval = xmap[x];
+	T newval = T( section[ index++ ] );
+	if ( !Util::is_nan( oldval ) && !Util::is_nan( newval ) ) {
+	  maxerr = Util::max( maxerr, fabs( ftype( newval - oldval ) ) );
+          newval = Util::max( oldval, newval );
+        }
+	xmap[x] = newval;
+      }
+    }
+  }
+
+  // done
+  CMap_io::ccp4_cmap_close( file );
+
+  // ASU checks
+  typedef Xmap_base::Map_reference_index MRI;
+  ASUerror asuerr = ASUCORRECT;
+  ftype s0(0.0), s1(0.0), s2(0.0);
+  for ( MRI ix = xmap.first(); !ix.last(); ix.next() ) {
+    T val = xmap[ix];
+    if ( Util::is_null( val ) ) {
+      asuerr = ASUINCOMPLETE;
+      xmap[ix] = missing;
+    } else {
+      s0 += 1.0;
+      s1 += val;
+      s2 += val*val;
+    }
+  }
+  if ( s0 > 0.5 ) s2 = sqrt( s2*s0 - s1*s1 ) / s0;
+  if ( maxerr > 0.01*s2 ) asuerr = ASUINCONSISTENT;
+
+  return asuerr;
+}
+
 
 // instantiate the template functions for all reasonable types
 // xmap import
@@ -500,6 +605,8 @@ template void CCP4MAPfile::export_nxmap<unsigned int>( const NXmap<unsigned int>
 template void CCP4MAPfile::export_nxmap<int>( const NXmap<int>& nxmap );
 template void CCP4MAPfile::export_nxmap<ftype32>( const NXmap<ftype32>& nxmap );
 template void CCP4MAPfile::export_nxmap<ftype64>( const NXmap<ftype64>& nxmap );
-
+// xmap import and check
+template CCP4MAPfile::ASUerror CCP4MAPfile::import_xmap_check_asu<ftype32>( Xmap<ftype32>& xmap, ftype32 missing ) const;
+template CCP4MAPfile::ASUerror CCP4MAPfile::import_xmap_check_asu<ftype64>( Xmap<ftype64>& xmap, ftype64 missing ) const;
 
 } // namespace clipper
