@@ -155,6 +155,66 @@ RTop_orth::RTop_orth( const std::vector<Coord_orth>& src, const std::vector<Coor
   (*this) = RTop_orth( rot, tgt_cen - rot*src_cen );
 }
 
+/*! Construct the operator which give the least-squares fit of one set
+  of coordinates onto another. The coodinates are stored as STL
+  vectors of Coord_orth. The lists must be the same size, and each
+  atom in the source list must correspond to the same atom in the
+  target list.  The algorithm employed is that of Kearsley,
+  S.K. (1989) 'On the orthogonal transformation used for structural
+  comparisons'. Acta Cryst. A45, 208-210.
+  \param src The source list (i.e. the atoms to be transformed).
+  \param tgt The target list (i.e. the fixed atoms).
+  \param wgt The weight to apply to each atom. */
+  RTop_orth::RTop_orth( const std::vector<Coord_orth>& src, const std::vector<Coord_orth>& tgt, const std::vector<ftype>& wgt )
+{
+  if ( src.size() != wgt.size() || tgt.size() != wgt.size() )
+    Message::message(Message_fatal("RTop_orth: coordinate list size mismatch"));
+  // get centre of mass
+  Coord_orth s, t, p, m;
+  Coord_orth src_cen(0.0,0.0,0.0);
+  Coord_orth tgt_cen(0.0,0.0,0.0);
+  ftype w = 0.0;
+  int n = src.size();
+  for ( int i = 0; i < n; i++ ) {
+    w += wgt[i];
+    src_cen += wgt[i] * src[i];
+    tgt_cen += wgt[i] * tgt[i];
+  }
+  src_cen = (1.0/w) * src_cen;
+  tgt_cen = (1.0/w) * tgt_cen;
+  // prepare cross-sums
+  Matrix<> mat( 4, 4, 0.0 );
+  for ( int i = 0; i < n; i++ ) {
+    w = wgt[i];
+    s = src[i] - src_cen;
+    t = tgt[i] - tgt_cen;
+    p = s + t;
+    m = s - t;
+    mat(0,0) = mat(0,0) + w * ( m[0]*m[0] + m[1]*m[1] + m[2]*m[2] );
+    mat(1,1) = mat(1,1) + w * ( m[0]*m[0] + p[1]*p[1] + p[2]*p[2] );
+    mat(2,2) = mat(2,2) + w * ( p[0]*p[0] + m[1]*m[1] + p[2]*p[2] );
+    mat(3,3) = mat(3,3) + w * ( p[0]*p[0] + p[1]*p[1] + m[2]*m[2] );
+    mat(0,1) = mat(0,1) + w * ( m[2]*p[1] - m[1]*p[2] );
+    mat(0,2) = mat(0,2) + w * ( m[0]*p[2] - m[2]*p[0] );
+    mat(0,3) = mat(0,3) + w * ( m[1]*p[0] - m[0]*p[1] );
+    mat(1,2) = mat(1,2) + w * ( m[0]*m[1] - p[0]*p[1] );
+    mat(1,3) = mat(1,3) + w * ( m[0]*m[2] - p[0]*p[2] );
+    mat(2,3) = mat(2,3) + w * ( m[1]*m[2] - p[1]*p[2] );
+  }
+  mat(1,0) = mat(0,1);
+  mat(2,0) = mat(0,2);
+  mat(2,1) = mat(1,2);
+  mat(3,0) = mat(0,3);
+  mat(3,1) = mat(1,3);
+  mat(3,2) = mat(2,3);
+  // eigenvalue calc
+  std::vector<ftype> v = mat.eigen( true );
+  // get result
+  Rotation r( mat(0,0), mat(1,0), mat(2,0), mat(3,0) );
+  Mat33<> rot = r.norm().matrix();
+  (*this) = RTop_orth( rot, tgt_cen - rot*src_cen );
+}
+
 /*! \param cell The cell concerned \return The transformed coordinate. */
 RTop_frac RTop_orth::rtop_frac( const Cell& cell ) const
 {
