@@ -261,7 +261,10 @@ namespace clipper
     int data_size() const { return T::data_size(); }
     String data_names() const { return T::data_names(); }
     void data_export( const HKL& hkl, xtype array[] ) const
-      { T datum; get_data( hkl, datum ); datum.data_export( array ); }
+      { T datum; const int index = hkl_info().index_of( hkl );
+        if ( index != -1 ) datum = list[index];
+        else               get_data( hkl, datum );
+        datum.data_export( array ); }
     void data_import( const HKL& hkl, const xtype array[] )
       { T datum; datum.data_import( array ); set_data( hkl, datum ); }
     void mask(const HKL_data_base& mask);
@@ -418,7 +421,7 @@ namespace clipper
   template<class T> void HKL_data<T>::mask(const HKL_data_base& mask)
   {
     T null; null.set_null();
-    for ( int i = 0; i < list.size(); i++ )
+    for ( unsigned int i = 0; i < list.size(); i++ )
       if ( mask.missing(i) ) list[i] = null;
   }
 
@@ -548,7 +551,7 @@ namespace clipper
     \param value The value to which the list is to be set. */
   template<class T> HKL_data<T>& HKL_data<T>::operator =( const T& value )
   {
-    for ( int i = 0; i < list.size(); i++ ) list[i] = value;
+    for ( unsigned int i = 0; i < list.size(); i++ ) list[i] = value;
     return *this;
   }
 
