@@ -144,6 +144,49 @@ namespace clipper {
   };
 
 
+  /*! This class implements the target function for calculating the
+    scale factor to scale one set of I's to another. The resulting
+    scale is the square of the factor that scales the first set of
+    data to match the second. */
+  template<class T1, class T2> class TargetFn_scaleI1I2 : public TargetFn_base
+  {
+  public:
+    //! constructor: takes the datalist against which to calc target
+    TargetFn_scaleI1I2( const HKL_data<T1>& hkl_data1_, const HKL_data<T2>& hkl_data2_ );
+    //! return the value and derivatives of the target function
+    Rderiv rderiv( const HKL_info::HKL_reference_index& ih, const ftype& fh ) const;
+    //! the type of the function: optionally used to improve convergence
+    FNtype type() const { return QUADRATIC; }
+  private:
+    const HKL_data<T1>* hkl_data1;
+    const HKL_data<T2>* hkl_data2;
+  };
+
+
+  //! log |I| scaling target
+  /*! This class implements the target function for calculating the
+    scale factor to scale the weighted log of one set of I's to
+    another. The resulting scale is the square of the factor that
+    scales the first set of data to match the second. The log scaling
+    target is used in conjunction with the log-Gaussian basis
+    functions for a fast and robust approximation to iso/aniso
+    Gaussian scaling.
+  */
+  template<class T1, class T2> class TargetFn_scaleLogI1I2 : public TargetFn_base
+  {
+  public:
+    //! constructor: takes the datalist against which to calc target
+    TargetFn_scaleLogI1I2( const HKL_data<T1>& hkl_data1_, const HKL_data<T2>& hkl_data2_ );
+    //! return the value and derivatives of the target function
+    Rderiv rderiv( const HKL_info::HKL_reference_index& ih, const ftype& fh ) const;
+    //! the type of the function: optionally used to improve convergence
+    FNtype type() const { return QUADRATIC; }
+  private:
+    const HKL_data<T1>* hkl_data1;
+    const HKL_data<T2>* hkl_data2;
+  };
+
+
   //! |E|<sup>2</sup> scaling target
   /*! This class implements the target function for calculating the
     scale factor to normalise to <|E|<sup>2</sup>> = 1. Note that this
@@ -335,6 +378,61 @@ namespace clipper {
     return result;
   }
 
+  // I1-I2 scaling
+
+  template<class T1, class T2> TargetFn_scaleI1I2<T1,T2>::TargetFn_scaleI1I2( const HKL_data<T1>& hkl_data1_, const HKL_data<T2>& hkl_data2_ )
+  {
+    hkl_data1 = &hkl_data1_;
+    hkl_data2 = &hkl_data2_;
+  }
+
+  template<class T1, class T2> TargetFn_base::Rderiv TargetFn_scaleI1I2<T1,T2>::rderiv( const HKL_info::HKL_reference_index& ih, const ftype& fh ) const
+  {
+    Rderiv result;
+    const T1& ft1 = (*hkl_data1)[ih];
+    const T2& ft2 = (*hkl_data2)[ih];
+    if ( !ft1.missing() && !ft2.missing() ) {
+      const ftype eps = ih.hkl_class().epsilon();
+      const ftype f1 = ft1.I() / eps;
+      const ftype f2 = ft2.I() / eps;
+      const ftype d = fh*f1 - f2;
+      result.r = d * d / f1;
+      result.dr = 2.0 * d;
+      result.dr2 = 2.0 * f1;
+    } else {
+      result.r = result.dr = result.dr2 = 0.0;
+    }
+    return result;
+  }
+
+  // Log I1-I2 scaling
+
+  template<class T1, class T2> TargetFn_scaleLogI1I2<T1,T2>::TargetFn_scaleLogI1I2( const HKL_data<T1>& hkl_data1_, const HKL_data<T2>& hkl_data2_ )
+  {
+    hkl_data1 = &hkl_data1_;
+    hkl_data2 = &hkl_data2_;
+  }
+
+  template<class T1, class T2> TargetFn_base::Rderiv TargetFn_scaleLogI1I2<T1,T2>::rderiv( const HKL_info::HKL_reference_index& ih, const ftype& fh ) const
+  {
+    Rderiv result;
+    result.r = result.dr = result.dr2 = 0.0;
+    const T1& ft1 = (*hkl_data1)[ih];
+    const T2& ft2 = (*hkl_data2)[ih];
+    if ( !ft1.missing() && !ft2.missing() )
+      if ( ft1.I() > 1.0e-6 && ft2.I() > 1.0e-6 ) {
+	const ftype eps = ih.hkl_class().epsilon();
+	const ftype f1 = ft1.I() / eps;
+	const ftype f2 = ft2.I() / eps;
+	const ftype w = 1.0;  // f1 * f2;
+	const ftype d = fh + log(f1) - log(f2);
+	result.r   =       w * d * d;
+	result.dr  = 2.0 * w * d;
+	result.dr2 = 2.0 * w;
+    }
+    return result;
+  }
+
   // E^2 scaling
 
   template<class T> TargetFn_scaleEsq<T>::TargetFn_scaleEsq( const HKL_data<T>& hkl_data_ )
@@ -425,7 +523,7 @@ namespace clipper {
     } else {
       ftype eo = eodata[ih].E();
       ftype ec = ecdata[ih].E();
-      ftype sigmaa = Util::min( Util::max( sigmaa0, 0.01 ), 0.99 );
+      ftype sigmaa = (sigmaa0>0.99)?(0.99):((sigmaa0<0.01)?0.01:sigmaa0); 
       ftype dx = 2.0 * eo * ec;
       ftype x = dx * sigmaa/(1-sigmaa*sigmaa);
       ftype t0 = 1.0/(1-sigmaa*sigmaa) + 0.5*log((1-sigmaa*sigmaa));
