@@ -667,8 +667,7 @@ void Spgr_descr::set_preferred( const char& c )
 
 Mutex Spgr_cacheobj::mutex = Mutex();
 
-struct Compare_grid{ bool operator() ( const Vec3<>& c1, const Vec3<>& c2 ) const { return (c1[0]*c1[1]*c1[2]+0.001*c1[1]+0.00001*c1[0]) <= (c2[0]*c2[1]*c2[
-2]+0.001*c2[1]+0.00001*c2[0]); } };
+struct Compare_grid{ bool operator() ( const Vec3<>& c1, const Vec3<>& c2 ) const { return (double(c1[0])*c1[1]*c1[2]+0.001*c1[1]+0.00001*c1[0]) < (double(c2[0])*c2[1]*c2[2]+0.001*c2[1]+0.00001*c2[0]); } };
 
 bool reci_asu( const Spgr_descr::Symop_codes& ops, data::ASUfn asufn )
 {
